@@ -1,4 +1,7 @@
 import asyncio
+import contextvars
+import functools
+import sys
 
 import pytest
 from click.testing import CliRunner
@@ -19,13 +22,28 @@ from distributed.compatibility import LINUX
 from distributed.deploy.utils import nprocesses_nthreads
 from distributed.metrics import time
 from distributed.utils import parse_ports, sync
-from distributed.utils_test import (
-    gen_cluster,
-    popen,
-    requires_ipv6,
-    terminate_process,
-    wait_for_port,
-)
+from distributed.utils_test import gen_cluster, popen, requires_ipv6
+
+if sys.version_info >= (3, 9):
+    from asyncio import to_thread
+else:
+
+    async def to_thread(*func_args, **kwargs):
+        """Asynchronously run function *func* in a separate thread.
+        Any *args and **kwargs supplied for this function are directly passed
+        to *func*. Also, the current :class:`contextvars.Context` is propagated,
+        allowing context variables from the main thread to be accessed in the
+        separate thread.
+        Return a coroutine that can be awaited to get the eventual result of *func*.
+
+        backport from
+        https://github.com/python/cpython/blob/3f1ea163ea54513e00e0e9d5442fee1b639825cc/Lib/asyncio/threads.py#L12-L25
+        """
+        func, *args = func_args
+        loop = asyncio.get_running_loop()
+        ctx = contextvars.copy_context()
+        func_call = functools.partial(ctx.run, func, *args, **kwargs)
+        return await loop.run_in_executor(None, func_call)
 
 
 def test_nanny_worker_ports(loop):
@@ -61,15 +79,15 @@ def test_nanny_worker_ports(loop):
 @pytest.mark.slow
 def test_nanny_worker_port_range(loop):
     with popen(["dask-scheduler", "--port", "9359", "--no-dashboard"]) as sched:
-        nprocs = 3
+        n_workers = 3
         worker_port = "9684:9686"
         nanny_port = "9688:9690"
         with popen(
             [
                 "dask-worker",
                 "127.0.0.1:9359",
-                "--nprocs",
-                f"{nprocs}",
+                "--nworkers",
+                f"{n_workers}",
                 "--host",
                 "127.0.0.1",
                 "--worker-port",
@@ -81,7 +99,7 @@ def test_nanny_worker_port_range(loop):
         ):
             with Client("127.0.0.1:9359", loop=loop) as c:
                 start = time()
-                while len(c.scheduler_info()["workers"]) < nprocs:
+                while len(c.scheduler_info()["workers"]) < n_workers:
                     sleep(0.1)
                     assert time() - start < 60
 
@@ -103,7 +121,7 @@ def test_nanny_worker_port_range_too_many_workers_raises(loop):
             [
                 "dask-worker",
                 "127.0.0.1:9359",
-                "--nprocs",
+                "--nworkers",
                 "3",
                 "--host",
                 "127.0.0.1",
@@ -149,24 +167,60 @@ def test_no_nanny(loop):
 
 @pytest.mark.slow
 @pytest.mark.parametrize("nanny", ["--nanny", "--no-nanny"])
-def test_no_reconnect(nanny, loop):
-    with popen(["dask-scheduler", "--no-dashboard"]) as sched:
-        wait_for_port(("127.0.0.1", 8786))
+@pytest.mark.asyncio
+async def test_no_reconnect(nanny):
+    async with Scheduler(dashboard_address=":0") as s, Client(
+        s.address, asynchronous=True
+    ) as c:
         with popen(
             [
                 "dask-worker",
-                "tcp://127.0.0.1:8786",
+                s.address,
                 "--no-reconnect",
                 nanny,
                 "--no-dashboard",
             ]
         ) as worker:
-            sleep(2)
-            terminate_process(sched)
-        start = time()
-        while worker.poll() is None:
-            sleep(0.1)
-            assert time() < start + 30
+            # roundtrip works
+            assert await c.submit(lambda x: x + 1, 10) == 11
+
+            (comm,) = s.stream_comms.values()
+            comm.abort()
+
+            # worker terminates as soon as the connection is aborted
+            await to_thread(worker.communicate, timeout=5)
+            assert worker.returncode == 0
+
+
+@pytest.mark.slow
+@pytest.mark.parametrize("nanny", ["--nanny", "--no-nanny"])
+@pytest.mark.asyncio
+async def test_reconnect(nanny):
+    async with Scheduler(dashboard_address=":0") as s, Client(
+        s.address, asynchronous=True
+    ) as c:
+        with popen(
+            [
+                "dask-worker",
+                s.address,
+                "--reconnect",
+                nanny,
+                "--no-dashboard",
+            ]
+        ) as worker:
+            # roundtrip works
+            await c.submit(lambda x: x + 1, 10) == 11
+
+            (comm,) = s.stream_comms.values()
+            comm.abort()
+
+            # roundtrip still works, which means the worker reconnected
+            assert await c.submit(lambda x: x + 1, 11) == 12
+
+            # closing the scheduler cleanly does terminate the worker
+            await s.close()
+            await to_thread(worker.communicate, timeout=5)
+            assert worker.returncode == 0
 
 
 def test_resources(loop):
@@ -237,10 +291,10 @@ def test_scheduler_address_env(loop, monkeypatch):
                     assert time() < start + 10
 
 
-def test_nprocs_requires_nanny(loop):
+def test_nworkers_requires_nanny(loop):
     with popen(["dask-scheduler", "--no-dashboard"]):
         with popen(
-            ["dask-worker", "127.0.0.1:8786", "--nprocs=2", "--no-nanny"]
+            ["dask-worker", "127.0.0.1:8786", "--nworkers=2", "--no-nanny"]
         ) as worker:
             assert any(
                 b"Failed to launch worker" in worker.stderr.readline()
@@ -248,25 +302,25 @@ def test_nprocs_requires_nanny(loop):
             )
 
 
-def test_nprocs_negative(loop):
+def test_nworkers_negative(loop):
     with popen(["dask-scheduler", "--no-dashboard"]):
-        with popen(["dask-worker", "127.0.0.1:8786", "--nprocs=-1"]):
+        with popen(["dask-worker", "127.0.0.1:8786", "--nworkers=-1"]):
             with Client("tcp://127.0.0.1:8786", loop=loop) as c:
                 c.wait_for_workers(cpu_count(), timeout="10 seconds")
 
 
-def test_nprocs_auto(loop):
+def test_nworkers_auto(loop):
     with popen(["dask-scheduler", "--no-dashboard"]):
-        with popen(["dask-worker", "127.0.0.1:8786", "--nprocs=auto"]):
+        with popen(["dask-worker", "127.0.0.1:8786", "--nworkers=auto"]):
             with Client("tcp://127.0.0.1:8786", loop=loop) as c:
                 procs, _ = nprocesses_nthreads()
                 c.wait_for_workers(procs, timeout="10 seconds")
 
 
-def test_nprocs_expands_name(loop):
+def test_nworkers_expands_name(loop):
     with popen(["dask-scheduler", "--no-dashboard"]):
-        with popen(["dask-worker", "127.0.0.1:8786", "--nprocs", "2", "--name", "0"]):
-            with popen(["dask-worker", "127.0.0.1:8786", "--nprocs", "2"]):
+        with popen(["dask-worker", "127.0.0.1:8786", "--nworkers", "2", "--name", "0"]):
+            with popen(["dask-worker", "127.0.0.1:8786", "--nworkers", "2"]):
                 with Client("tcp://127.0.0.1:8786", loop=loop) as c:
                     start = time()
                     while len(c.scheduler_info()["workers"]) < 4:
@@ -280,6 +334,30 @@ def test_nprocs_expands_name(loop):
                     assert len(set(names)) == 4
 
 
+def test_worker_cli_nprocs_renamed_to_nworkers(loop):
+    n_workers = 2
+    with popen(["dask-scheduler", "--no-dashboard"]):
+        with popen(
+            ["dask-worker", "127.0.0.1:8786", f"--nprocs={n_workers}"]
+        ) as worker:
+            assert any(
+                b"renamed to --nworkers" in worker.stderr.readline() for i in range(15)
+            )
+            with Client("tcp://127.0.0.1:8786", loop=loop) as c:
+                c.wait_for_workers(n_workers, timeout="30 seconds")
+
+
+def test_worker_cli_nworkers_with_nprocs_is_an_error():
+    with popen(["dask-scheduler", "--no-dashboard"]):
+        with popen(
+            ["dask-worker", "127.0.0.1:8786", "--nprocs=2", "--nworkers=2"]
+        ) as worker:
+            assert any(
+                b"Both --nprocs and --nworkers" in worker.stderr.readline()
+                for i in range(15)
+            )
+
+
 @pytest.mark.skipif(not LINUX, reason="Need 127.0.0.2 to mean localhost")
 @pytest.mark.parametrize("nanny", ["--nanny", "--no-nanny"])
 @pytest.mark.parametrize(
