@@ -29,6 +29,7 @@ from distributed.utils_test import (
 pytestmark = pytest.mark.ci1
 
 
+@pytest.mark.slow()
 def test_submit_after_failed_worker_sync(loop):
     with cluster() as (s, [a, b]):
         with Client(s["address"], loop=loop) as c:
@@ -39,6 +40,7 @@ def test_submit_after_failed_worker_sync(loop):
             assert total.result() == sum(map(inc, range(10)))
 
 
+@pytest.mark.slow()
 @gen_cluster(client=True, timeout=60, active_rpc_timeout=10)
 async def test_submit_after_failed_worker_async(c, s, a, b):
     n = await Nanny(s.address, nthreads=2, loop=s.loop)
@@ -99,43 +101,6 @@ async def test_gather_then_submit_after_failed_workers(c, s, w, x, y, z):
         assert result == [sum(map(inc, range(20)))]
 
 
-@pytest.mark.xfail(COMPILED, reason="Fails with cythonized scheduler")
-@gen_cluster(Worker=Nanny, client=True, timeout=60)
-async def test_failed_worker_without_warning(c, s, a, b):
-    L = c.map(inc, range(10))
-    await wait(L)
-
-    original_pid = a.pid
-    with suppress(CommClosedError):
-        await c._run(os._exit, 1, workers=[a.worker_address])
-    start = time()
-    while a.pid == original_pid:
-        await asyncio.sleep(0.01)
-        assert time() - start < 10
-
-    await asyncio.sleep(0.5)
-
-    start = time()
-    while len(s.nthreads) < 2:
-        await asyncio.sleep(0.01)
-        assert time() - start < 10
-
-    await wait(L)
-
-    L2 = c.map(inc, range(10, 20))
-    await wait(L2)
-    assert all(len(keys) > 0 for keys in s.has_what.values())
-    nthreads2 = dict(s.nthreads)
-
-    await c.restart()
-
-    L = c.map(inc, range(10))
-    await wait(L)
-    assert all(len(keys) > 0 for keys in s.has_what.values())
-
-    assert not (set(nthreads2) & set(s.nthreads))  # no overlap
-
-
 @pytest.mark.xfail(COMPILED, reason="Fails with cythonized scheduler")
 @gen_cluster(Worker=Nanny, client=True, timeout=60)
 async def test_restart(c, s, a, b):
