@@ -3,9 +3,9 @@ import os
 import shutil
 import sys
 import tempfile
-import time
 import urllib.error
 import urllib.request
+from time import sleep
 
 import pytest
 import tornado
@@ -15,6 +15,7 @@ import dask
 
 from distributed import Client, Nanny, Scheduler, Worker
 from distributed.compatibility import MACOS
+from distributed.metrics import time
 from distributed.utils_test import captured_logger, cluster, gen_cluster, gen_test
 
 PY_VERSION = sys.version_info[:2]
@@ -179,13 +180,13 @@ def create_preload_application():
 def scheduler_preload():
     p = multiprocessing.Process(target=create_preload_application)
     p.start()
-    start = time.time()
+    start = time()
     while not p.is_alive():
-        if time.time() > start + 5:
+        if time() > start + 5:
             raise AssertionError("Process didn't come up")
-        time.sleep(0.5)
+        sleep(0.5)
     # Make sure we can query the server
-    start = time.time()
+    start = time()
     request = urllib.request.Request("http://127.0.0.1:12345/preload", method="GET")
     while True:
         try:
@@ -193,9 +194,9 @@ def scheduler_preload():
             if response.status == 200:
                 break
         except urllib.error.URLError as e:
-            if time.time() > start + 10:
+            if time() > start + 10:
                 raise AssertionError("Webserver didn't come up", e)
-            time.sleep(0.5)
+            sleep(0.5)
 
     yield
     p.kill()
@@ -256,23 +257,23 @@ def create_worker_preload_application():
 def worker_preload():
     p = multiprocessing.Process(target=create_worker_preload_application)
     p.start()
-    start = time.time()
+    start = time()
     while not p.is_alive():
-        if time.time() > start + 5:
+        if time() > start + 5:
             raise AssertionError("Process didn't come up")
-        time.sleep(0.5)
+        sleep(0.5)
     # Make sure we can query the server
     request = urllib.request.Request("http://127.0.0.1:12346/preload", method="GET")
-    start = time.time()
+    start = time()
     while True:
         try:
             response = urllib.request.urlopen(request)
             if response.status == 200:
                 break
         except urllib.error.URLError as e:
-            if time.time() > start + 10:
+            if time() > start + 10:
                 raise AssertionError("Webserver didn't come up", e)
-            time.sleep(0.5)
+            sleep(0.5)
 
     yield
     p.kill()
